@@ -1031,7 +1031,8 @@ bool QOpenGLContext::makeCurrent(QSurface *surface)
                         || qstrcmp(rendererString, "GC800 core") == 0
                         || qstrcmp(rendererString, "GC1000 core") == 0
                         || strstr(rendererString, "GC2000") != nullptr
-                        || qstrcmp(rendererString, "Immersion.16") == 0;
+                        || qstrcmp(rendererString, "Immersion.16") == 0
+                        || qstrncmp(rendererString, "Apple Mx", 7) == 0;
         }
         needsWorkaroundSet = true;
     }
