@@ -146,7 +146,7 @@ QT_BEGIN_NAMESPACE
 */
 
 /*!
-    Contructs a QGraphicsLayout object.
+    Constructs a QGraphicsLayout object.
 
     \a parent is passed to QGraphicsLayoutItem's constructor and the
     QGraphicsLayoutItem's isLayout argument is set to \e true.
@@ -338,7 +338,7 @@ void QGraphicsLayout::invalidate()
             }
             if (layoutItem && !layoutItem->isLayout()) {
                 // If a layout has a parent that is not a layout it must be a QGraphicsWidget.
-                QApplication::postEvent(static_cast<QGraphicsWidget *>(layoutItem), new QEvent(QEvent::LayoutRequest));
+                QCoreApplication::postEvent(static_cast<QGraphicsWidget *>(layoutItem), new QEvent(QEvent::LayoutRequest));
             }
         }
     }
