@@ -69,6 +69,7 @@ private slots:
     void initializerList();
     void eraseValidIteratorOnSharedHash();
     void equal_range();
+    void insert_hash();
 };
 
 struct IdentityTracker {
@@ -273,7 +274,7 @@ void tst_QHash::insert1()
     {
         typedef QHash<QString, QString> Hash;
         Hash hash;
-        QString key;
+        QString key = QLatin1String("  ");
         for (int i = 0; i < 10; ++i) {
             key[0] = i + '0';
             for (int j = 0; j < 10; ++j) {
@@ -1143,6 +1144,12 @@ void tst_QHash::keyValueIterator()
 
         entry_type pair(it.key(), it.value());
         QCOMPARE(*key_value_it, pair);
+        QCOMPARE(key_value_it->first, pair.first);
+        QCOMPARE(key_value_it->second, pair.second);
+        QCOMPARE(&(*key_value_it).first, &it.key());
+        QCOMPARE(&key_value_it->first,   &it.key());
+        QCOMPARE(&(*key_value_it).second, &it.value());
+        QCOMPARE(&key_value_it->second,   &it.value());
         ++key_value_it;
         ++it;
     }
@@ -1480,7 +1487,6 @@ void tst_QHash::twoArguments_qHash()
 
 void tst_QHash::initializerList()
 {
-#ifdef Q_COMPILER_INITIALIZER_LISTS
     QHash<int, QString> hash = {{1, "bar"}, {1, "hello"}, {2, "initializer_list"}};
     QCOMPARE(hash.count(), 2);
     QCOMPARE(hash[1], QString("hello"));
@@ -1507,9 +1513,6 @@ void tst_QHash::initializerList()
 
     QMultiHash<int, float> emptyPairs2{{}, {}};
     QVERIFY(!emptyPairs2.isEmpty());
-#else
-    QSKIP("Compiler doesn't support initializer lists");
-#endif
 }
 
 void tst_QHash::eraseValidIteratorOnSharedHash()
@@ -1647,5 +1650,77 @@ void tst_QHash::equal_range()
     }
 }
 
+void tst_QHash::insert_hash()
+{
+    {
+        QHash<int, int> hash;
+        hash.insert(1, 1);
+        hash.insert(2, 2);
+        hash.insert(0, -1);
+
+        QHash<int, int> hash2;
+        hash2.insert(0, 0);
+        hash2.insert(3, 3);
+        hash2.insert(4, 4);
+
+        hash.insert(hash2);
+
+        QCOMPARE(hash.count(), 5);
+        for (int i = 0; i < 5; ++i)
+            QCOMPARE(hash[i], i);
+    }
+    {
+        QHash<int, int> hash;
+        hash.insert(0, 5);
+
+        QHash<int, int> hash2;
+
+        hash.insert(hash2);
+
+        QCOMPARE(hash.count(), 1);
+        QCOMPARE(hash[0], 5);
+    }
+    {
+        QHash<int, int> hash;
+        QHash<int, int> hash2;
+        hash2.insert(0, 5);
+
+        hash.insert(hash2);
+
+        QCOMPARE(hash.count(), 1);
+        QCOMPARE(hash[0], 5);
+        QCOMPARE(hash, hash2);
+    }
+    {
+        QHash<int, int> hash;
+        hash.insert(0, 7);
+        hash.insert(2, 5);
+        hash.insert(7, 55);
+
+        // insert into ourself, nothing should happen
+        hash.insert(hash);
+
+        QCOMPARE(hash.count(), 3);
+        QCOMPARE(hash[0], 7);
+        QCOMPARE(hash[2], 5);
+        QCOMPARE(hash[7], 55);
+    }
+    {
+        // This will use a QMultiHash and then insert that into QHash,
+        // the ordering is undefined so we won't test that but make
+        // sure this isn't adding multiple entries with the same key
+        // to the QHash.
+        QHash<int, int> hash;
+        QMultiHash<int, int> hash2;
+        hash2.insert(0, 5);
+        hash2.insert(0, 6);
+        hash2.insert(0, 7);
+
+        hash.insert(hash2);
+
+        QCOMPARE(hash.count(), 1);
+    }
+}
+
 QTEST_APPLESS_MAIN(tst_QHash)
 #include "tst_qhash.moc"
